package servlets;

/**
 * Sample Frogans Site for the Java FsdlRequest Parser
 *
 * @version		0.3
 *
 * @copyright	1999-2018 OP3FT
 *
 * This Java servlet, along with 'result.jsp', generates an FSDL
 * document. It is part of a sample Frogans site provided by the
 * OP3FT to help youcreate your own dynamic Frogans sites based on
 * the Java FsdlRequest parser.
 *
 * You are free to use, copy, distribute, modify, adapt, or share this
 * sample Frogans site.
 *
 * @file		Result.java
 */


import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import fsdl_request_parser.FsdlRequest;
import fsdl_request_parser.FsdlRequestNavigation;
import fsdl_request_parser.FsdlRequestWanted;


public class Result extends HttpServlet {

	private static final long serialVersionUID 					= 1L;
	public static final String RESULT_VIEW        				= "/WEB-INF/result.jsp";
	public static final String FSDL_REQUEST_DOCUMENT_VERSION 	= "3.0";

	public String escapeFieldValue(String value)		// Escape "&", "<" and " " characters in a field value to be displayed
	{
		String tmp = value.replaceAll("&", "&amp;");
		tmp = tmp.replaceAll("<", "&lt;");
		tmp = tmp.replaceAll(" ", "&#xA0;");
		return tmp;
	}


	public void doGet( HttpServletRequest request, HttpServletResponse response )	throws ServletException, IOException {

		// This should never occur when navigating this sample Frogans site
		System.exit(1);
	}


	public void doPost( HttpServletRequest request, HttpServletResponse response )	throws ServletException, IOException {

		FsdlRequest fsdlRequest = new FsdlRequest();
		if (!fsdlRequest.initialize(FSDL_REQUEST_DOCUMENT_VERSION, request.getInputStream())) { // This should never occur when navigating this sample Frogans site
			System.out.print(fsdlRequest.getErrorMessage());
			System.exit(1);
		}
		else {

			// Getting all available data from fsdlRequest

			int fsdlRequestWanted = fsdlRequest.getFsdlRequestWanted();
				//	Value is always applicable
				//	Value is one of the following constants:
				//		FsdlRequestWanted.FSDL_DOCUMENT
				//		FsdlRequestWanted.AUXILIARY_FILE

			int fsdlRequestNavigation = fsdlRequest.getFsdlRequestNavigation();
				//	Value is applicable only if $fsdlRequestWanted is FsdlRequestWanted.FSDL_DOCUMENT
				//	If not applicable, value is FSDL_REQUEST_UNDEFINED
				//	If applicable, value is one of the following constants:
				//		FsdlRequestNavigation.BUTTON
				//		FsdlRequestNavigation.NEXT
				//		FsdlRequestNavigation.REDIRECT
				//		FsdlRequestNavigation.OPEN

			int fsdlRequestKind = fsdlRequest.getFsdlRequestKind();
				//	Value is applicable only if $fsdlRequestWanted is FsdlRequestWanted.AUXILIARY_FILE
				//	If not applicable, value is FSDL_REQUEST_UNDEFINED
				//	If applicable, possible values are the following constants:
				//		FsdlRequestKind.IMAGE


			HashMap<String,String> sessionFields = fsdlRequest.getSessionFields();
				//	Value is always applicable
				//	Value is a HashMap of up to FsdlRequest.SESSION_FIELDS_COUNT_MAX key-value pairs.
				//	The HashMap can be empty.

			List<HashMap<String,String>> sessionFieldsList = new ArrayList<HashMap<String,String>>();
			HashMap<String,String> fields;
			for (HashMap.Entry<String, String> entry : sessionFields.entrySet()) {
				fields = new HashMap<String, String>();
				fields.put("key", entry.getKey());
				fields.put("value", entry.getValue());
				sessionFieldsList.add(fields);
			}


			HashMap<String,String> fileFields = fsdlRequest.getFileFields();
				//	Value is applicable only if fsdlRequestWanted is FsdlRequestWanted.FSDL_DOCUMENT
				//									and fsdlRequestNavigation is not FsdlRequestNavigation::OPEN
				//						  or if fsdlRequestWanted is FsdlRequestWanted.AUXILIARY_FILE
				//	If not applicable, value is an empty HashMap.
				//	If applicable, value is a HashMap of up to FsdlRequest.FILE_FIELDS_COUNT_MAX key-value pairs.
				//	The HashMap can be empty.

			List<HashMap<String,String>> fileFieldsList = new ArrayList<HashMap<String,String>>();
			for (HashMap.Entry<String, String> entry : fileFields.entrySet()) {
				fields = new HashMap<String, String>();
				fields.put("key", entry.getKey());
				fields.put("value", entry.getValue());
				fileFieldsList.add(fields);
			}


			HashMap<String,String> entryFields = fsdlRequest.getEntryFields();
				//	Value is applicable only if $fsdlRequestWanted is FsdlRequestWanted.FSDL_DOCUMENT
				//									and $fsdlRequestNavigation is FsdlRequestNavigation::BUTTON
				//	If not applicable, value is an empty HashMap.
				//	If applicable, value is a HashMap of up to FsdlRequest.ENTRY_FIELDS_COUNT_MAX key-value pairs.
				//	The HashMap can be empty.

			List<HashMap<String,String>> entryFieldsList = new ArrayList<HashMap<String,String>>();
			for (HashMap.Entry<String, String> entry : entryFields.entrySet()) {
				fields = new HashMap<String, String>();
				fields.put("key", entry.getKey());
				fields.put("value", entry.getValue());
				entryFieldsList.add(fields);
			}

			// Check request context

			if (fsdlRequestWanted != FsdlRequestWanted.FSDL_DOCUMENT)	// This should never occur when navigating this sample Frogans site
			{
				System.exit(1);
			}

			if (fsdlRequestNavigation != FsdlRequestNavigation.BUTTON)	// This should never occur when navigating this sample Frogans site
			{
				System.exit(1);
			}


			// Prepare content to be displayed on this result Frogans slide

			String fsdlRequestWantedText = FsdlRequestWanted.toString(fsdlRequestWanted);
			String fsdlRequestNavigationText = FsdlRequestNavigation.toString(fsdlRequestNavigation);
			String fsdlRequestKindText = FsdlRequestNavigation.toString(fsdlRequestKind);

			int sessionFieldsCount = sessionFields.size();
			int fileFieldsCount = fileFields.size();
			int entryFieldsCount = entryFields.size();

			if ((sessionFieldsCount + fileFieldsCount + entryFieldsCount) > 7)	// This should never occur when navigating this sample Frogans site
			{
				System.exit(1);
			}

			request.setAttribute( "fsdlRequestWantedText", fsdlRequestWantedText);
			request.setAttribute( "fsdlRequestNavigationText", fsdlRequestNavigationText);
			request.setAttribute( "fsdlRequestKindText", fsdlRequestKindText);
			request.setAttribute( "sessionFieldsCount", sessionFieldsCount);
			request.setAttribute( "fileFieldsCount", fileFieldsCount);
			request.setAttribute( "entryFieldsCount", entryFieldsCount);
			request.setAttribute( "sessionFieldsList", sessionFieldsList);
			request.setAttribute( "fileFieldsList", fileFieldsList);
			request.setAttribute( "entryFieldsList", entryFieldsList);

			this.getServletContext().getRequestDispatcher( RESULT_VIEW ).forward( request, response );

			// See additional JSP instructions embedded in 'result.jsp'.
		}
	}


}